#include "delayui.h"
#include "delay.h"
#include <stdio.h>

//-----------------------------------------------------------------------------
// resource id's
enum {
	// bitmaps
	kBackgroundId = 128,
	kFaderBodyId,
	kFaderHandleId,
	
	// positions
	kFaderX = 18,
	kFaderY = 10,

	kFaderInc = 18,

	kDisplayX = 10,
	kDisplayY = 184,
	kDisplayXWidth = 30,
	kDisplayHeight = 14
};

//-----------------------------------------------------------------------------
// prototype string convert float -> percent
void percentStringConvert (float value, char* string);
void percentStringConvert (float value, char* string)
{
	 sprintf (string, "%d%%", (int)(100 * value + 0.5f));
}


//-----------------------------------------------------------------------------
// SDEditor class implementation
//-----------------------------------------------------------------------------
DelayUI::DelayUI(AudioEffect *effect)
 : AEffGUIEditor (effect) 
{	
	delayFaderLeft = 0;
	delayFaderRight = 0;
	feedbackFaderLeft = 0;
	feedbackFaderRight = 0;
	delayDisplayLeft = 0;
	delayDisplayRight = 0;
	feedbackDisplayLeft = 0;
	feedbackDisplayRight = 0;

	// load the background bitmap
	// we don't need to load all bitmaps, this could be done when open is called
	hBackground = new CBitmap (kBackgroundId);

	// init the size of the plugin
	rect.left   = 0;
	rect.top    = 0;
	rect.right  = (short)hBackground->getWidth ();
	rect.bottom = (short)hBackground->getHeight ();
}

//-----------------------------------------------------------------------------
DelayUI::~DelayUI ()
{
	// free the background bitmap
	if (hBackground)
		hBackground->forget ();
	hBackground = 0;
}

//-----------------------------------------------------------------------------
bool DelayUI::open (void *ptr)
{	
	// !!! always call this !!!
	AEffGUIEditor::open (ptr);
	
	//--load some bitmaps
	CBitmap* hFaderBody   = new CBitmap (kFaderBodyId);
	CBitmap* hFaderHandle = new CBitmap (kFaderHandleId);

	//--init background frame-----------------------------------------------
	// We use a local CFrame object so that calls to setParameter won't call into objects which may not exist yet. 
	// If all GUI objects are created we assign our class member to this one. See bottom of this method.
	CRect size (0, 0, hBackground->getWidth (), hBackground->getHeight ());
	CFrame* lFrame = new CFrame (size, ptr, this);
	lFrame->setBackground (hBackground);

	//--init the faders------------------------------------------------
	int minPos = kFaderY;
	int maxPos = kFaderY + hFaderBody->getHeight () - hFaderHandle->getHeight () - 1;
	CPoint point (0, 0);
	CPoint offset (1, 0);

	// DelayLeft
	size (kFaderX, kFaderY,
          kFaderX + hFaderBody->getWidth (), kFaderY + hFaderBody->getHeight ());
	delayFaderLeft = new CVerticalSlider (size, this, kDelayLeft, minPos, maxPos, hFaderHandle, hFaderBody, point);
	delayFaderLeft->setOffsetHandle (offset);
	delayFaderLeft->setValue (effect->getParameter (kDelayLeft));
	lFrame->addView (delayFaderLeft);

	// DelayRight
	size.offset (kFaderInc + hFaderBody->getWidth (), 0);
	delayFaderRight = new CVerticalSlider (size, this, kDelayRight, minPos, maxPos, hFaderHandle, hFaderBody, point);
	delayFaderRight->setOffsetHandle (offset);
	delayFaderRight->setValue (effect->getParameter (kDelayRight));
	lFrame->addView (delayFaderRight);

	// FeedBackLeft
	size.offset (kFaderInc + hFaderBody->getWidth (), 0);
	feedbackFaderLeft = new CVerticalSlider (size, this, kFeedBackLeft, minPos, maxPos, hFaderHandle, hFaderBody, point);
	feedbackFaderLeft->setOffsetHandle (offset);
	feedbackFaderLeft->setValue (effect->getParameter (kFeedBackLeft));
	lFrame->addView (feedbackFaderLeft);

	// FeedBackRight
	size.offset (kFaderInc + hFaderBody->getWidth (), 0);
	feedbackFaderRight = new CVerticalSlider (size, this, kFeedBackRight, minPos, maxPos, hFaderHandle, hFaderBody, point);
	feedbackFaderRight->setOffsetHandle (offset);
	feedbackFaderRight->setValue (effect->getParameter (kFeedBackRight));
	lFrame->addView (feedbackFaderRight);


	//--init the display------------------------------------------------
	// DelayLeft
	size (kDisplayX, kDisplayY,
          kDisplayX + kDisplayXWidth, kDisplayY + kDisplayHeight);
	delayDisplayLeft = new CParamDisplay (size, 0, kCenterText);
	delayDisplayLeft->setFont (kNormalFontSmall);
	delayDisplayLeft->setFontColor (kWhiteCColor);
	delayDisplayLeft->setBackColor (kBlackCColor);
	delayDisplayLeft->setFrameColor (kBlueCColor);
	delayDisplayLeft->setValue (effect->getParameter (kDelayLeft));
	lFrame->addView (delayDisplayLeft);
	
	// DelayRight
	size.offset (kFaderInc + hFaderBody->getWidth (), 0);
	delayDisplayRight = new CParamDisplay (size, 0, kCenterText);
	delayDisplayRight->setFont (kNormalFontSmall);
	delayDisplayRight->setFontColor (kWhiteCColor);
	delayDisplayRight->setBackColor (kBlackCColor);
	delayDisplayRight->setFrameColor (kBlueCColor);
	delayDisplayRight->setValue (effect->getParameter (kDelayRight));
	lFrame->addView (delayDisplayRight);

	// FeedBackLeft
	size.offset (kFaderInc + hFaderBody->getWidth (), 0);
	feedbackDisplayLeft = new CParamDisplay (size, 0, kCenterText);
	feedbackDisplayLeft->setFont (kNormalFontSmall);
	feedbackDisplayLeft->setFontColor (kWhiteCColor);
	feedbackDisplayLeft->setBackColor (kBlackCColor);
	feedbackDisplayLeft->setFrameColor (kBlueCColor);
	feedbackDisplayLeft->setValue (effect->getParameter (kFeedBackLeft));
	feedbackDisplayLeft->setStringConvert (percentStringConvert);
	lFrame->addView (feedbackDisplayLeft);

	// FeedBackRight
	size.offset (kFaderInc + hFaderBody->getWidth (), 0);
	feedbackDisplayRight = new CParamDisplay (size, 0, kCenterText);
	feedbackDisplayRight->setFont (kNormalFontSmall);
	feedbackDisplayRight->setFontColor (kWhiteCColor);
	feedbackDisplayRight->setBackColor (kBlackCColor);
	feedbackDisplayRight->setFrameColor (kBlueCColor);
	feedbackDisplayRight->setValue (effect->getParameter (kFeedBackRight));
	feedbackDisplayRight->setStringConvert (percentStringConvert);
	lFrame->addView (feedbackDisplayRight);


	// Note : in the constructor of a CBitmap, the number of references is set to 1.
	// Then, each time the bitmap is used (for hinstance in a vertical slider), this
	// number is incremented.
	// As a consequence, everything happens as if the constructor by itself was adding
	// a reference. That's why we need til here a call to forget ().
	// You mustn't call delete here directly, because the bitmap is used by some CControls...
	// These "rules" apply to the other VSTGUI objects too.
	hFaderBody->forget ();
	hFaderHandle->forget ();

	frame = lFrame;
	return true;
}

//-----------------------------------------------------------------------------
void DelayUI::close ()
{
	delete frame;
	frame = 0;
}

//-----------------------------------------------------------------------------
void DelayUI::setParameter (VstInt32 index, float value)
{
	if (frame == 0)
		return;
	
	switch (index)
	{
		case kDelayLeft:
			if (delayFaderLeft) 
				delayFaderLeft->setValue (effect->getParameter (index));
			if (delayDisplayLeft)
				delayDisplayLeft->setValue (effect->getParameter (index));
			break;
		
		case kDelayRight:
			if (delayFaderRight)
				delayFaderLeft->setValue (effect->getParameter (index));
			if (delayDisplayRight)
				delayDisplayRight->setValue (effect->getParameter (index));
			break;

		case kFeedBackLeft:
			if (feedbackFaderLeft)
				feedbackFaderLeft->setValue (effect->getParameter (index));
			if (feedbackDisplayLeft)
				feedbackDisplayLeft->setValue (effect->getParameter (index));
			break;

		case kFeedBackRight:
			if (feedbackFaderRight)
				feedbackFaderRight->setValue (effect->getParameter (index));
			if (feedbackDisplayRight)
				feedbackDisplayRight->setValue (effect->getParameter (index));
			break;
		
	}
}

//-----------------------------------------------------------------------------
void DelayUI::valueChanged (CDrawContext* context, CControl* control)
{	
	long tag = control->getTag ();
	switch (tag)
	{
		case kDelayLeft:
			delayDisplayLeft->setValue (effect->getParameter (kDelayLeft));
			break;
		case kDelayRight:
			delayDisplayRight->setValue (effect->getParameter (kDelayRight));
			break;
		case kFeedBackLeft:
			feedbackDisplayLeft->setValue (effect->getParameter (kFeedBackLeft));
			break;
		case kFeedBackRight:
			feedbackDisplayRight->setValue (effect->getParameter (kFeedBackRight));
			break;
		break;
		
	}
	effect->setParameterAutomated (tag, control->getValue ());
	control->setDirty ();
}