package header;

import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * Methoden fr das Lesen und Schreiben des Headers von Wave-Dateien
 * @author Michael Steppat
 *
 */

public class WavFileHeader  {
	/**
	 * Schreibt einen short-Wert im Little-Endian-Format in eine Datei
	 * @param file			Referenz auf das RandomAccessFile-Objekt
	 * @param val			bergebener Wert vom Typ short
	 * @throws IOException	Wird bei Zugriffsproblemeen ausgelst
	 */
	private void writeShort(RandomAccessFile file, short val) throws IOException{
		file.writeByte(val);
		file.writeByte(val >> 8);		
	}
	/**
	 * Schreibt einen int-Wert im Little-Endian-Format in eine Datei
	 * @param file			Referenz auf das RandomAccessFile-Objekt
	 * @param val			bergebener Wert vom Typ int
	 * @throws IOException	Wird bei Zugriffsproblemen ausgelst
	 */
	private void writeInt(RandomAccessFile file, int val) throws IOException{
		file.writeByte(val);
		file.writeByte(val >> 8);
		file.writeByte(val >> 16);
		file.writeByte(val >> 24);		
	}
	/**
	 * Liest einen short-Wert an der Position des Filepointers
	 * @param file			Referenz auf das RandomAccessFile-Objekt
	 * @return				Ausgelesener Wert
	 * @throws IOException	Wird bei Zugriffsproblemen ausgelst
	 */
	private short readShort(RandomAccessFile file) throws IOException {
		short[] b = {0, 0};		
		b[0] =  file.readByte();
		b[1] =  file.readByte();		
		return (short)(0xff & b[0] | ((0xff & b[1]) << 8));		
	}
	/**
	 * Liest einen int-Wert an der Position des Filepointers
	 * @param file			Referenz auf das RandomAccessFile-Objekt
	 * @return				Ausgelesener Wert
	 * @throws IOException	Wird bei Zugriffsproblemen ausgelst
	 */
	private int readInt(RandomAccessFile file) throws IOException {
		byte[] b = {0, 0, 0, 0};		
		b[0] =  file.readByte();
		b[1] =  file.readByte();
		b[2] =  file.readByte();
		b[3] =  file.readByte();		
		return (0xff & b[0]) | ((0xff & b[1]) << 8) | ((0xff & b[2]) << 16) | ((0xff & b[3]) << 24);			
	}
	/**
	 * Sucht in der Datei nach einem bestimmten Chunk und positioniert den Filepointer an den Beginn der Chunkdaten
	 * @param file			Referenz auf das RandomAccessFile-Objekt
	 * @param chunk			Suchbegriff z.B. "RIFF", "WAVE", "fmt " usw.
	 * @return				Lnge des Chunks
	 * @throws IOException  Wird bei Zugriffsproblemen ausgelst
	 */
	private int findChunk(RandomAccessFile file, String chunk)  throws IOException {
		int searchChunk = 0;
		searchChunk = chunk.charAt(0) << 24 | chunk.charAt(1) << 16 | chunk.charAt(2) << 8 | chunk.charAt(3);
		int i;
		int len = 0;
		
		for(i = 0; i < file.length() - 4; i++) {
			if(file.readInt() == searchChunk) {
				// Lnge des Chunks
				len = readInt(file);
				break;
			}
			file.seek(file.getFilePointer() - 3);					
		}	
		return len;
	}
	/**
	 * Schreibt den Waveheader
	 * @param file			Referenz auf das RandomAccessFile-Objekt	
	 * @param waveFormatEx	Referenz auf das WaveFormatEx-Objekt
	 * @throws IOException	Wird bei Zugriffsproblemen ausgelst
	 */
	public void writeHeader(RandomAccessFile file, WaveFormatEx waveFormatEx) throws IOException{
		file.seek(0);
		byte[] chunk0 = {'R', 'I', 'F', 'F'};
		file.write(chunk0);
		int len = (int)file.length() - 8;
		writeInt(file, len);			
		byte[] chunk1 = {'W', 'A', 'V', 'E'};
		file.write(chunk1);
		byte[] chunk2 = {'f', 'm', 't', ' '};
		file.write(chunk2);
		len = 16;				// Lnge der auf den fmt -Chunk folgenden Daten
		writeInt(file, len);		
		writeShort(file, waveFormatEx.wFormatTag);		
		writeShort(file, waveFormatEx.nChannels);		
		writeInt(file, waveFormatEx.nSamplesPerSec);		
		writeInt(file, waveFormatEx.nAvgBytesPerSec);		
		writeShort(file, waveFormatEx.nBlockAlign);
		writeShort(file, waveFormatEx.wBitsPerSample);		
		byte[] chunk3 = {'d', 'a', 't', 'a'};
		file.write(chunk3);
		len = (int)file.length() - 44;
		writeInt(file, len);			
	}
	/**
	 * Liest den Waveheader einer Datei und fllt das WaveFormatEx-Objekt mit Daten
	 * @param file			Referenz auf das RandomAccessFile-Objekt
	 * @param waveFormatEx	Referenz auf das WaveFormatEx-Objekt
	 * @return				Lnge der Audiodaten bzw. des data-Chunks
	 * @throws IOException	Wird bei Zugriffsproblemen ausgelst
	 */
	
	public int readHeader(RandomAccessFile file, WaveFormatEx waveFormatEx) throws IOException{		
		file.seek(0);
		// Suchen des fmt-Chunks		
		findChunk(file, "fmt ");
		// Lesen des fmt-Chunks:
		waveFormatEx.wFormatTag = readShort(file);		
		waveFormatEx.nChannels = readShort(file);		
		waveFormatEx.nSamplesPerSec = readInt(file);		
		waveFormatEx.nAvgBytesPerSec = readInt(file);		
		waveFormatEx.nBlockAlign = readShort(file);
		waveFormatEx.wBitsPerSample = readShort(file);		
		// Suchen des data-Chunks:
		int len = findChunk(file, "data");				
		return len;
	}
}

